// File    : ULONGRV.CPP
// Author  : Eric Woodruff,  CIS ID: 72134,1150
// Updated : Tue 01/24/95 20:37:28
// Note    : Hereby declared public domain
// Compiler: Borland C++ 3.1 to 4.xx
//
// This is identical to the standard TRangeValidator class, but it will
// work with unsigned long values instead of signed long values.
//

#include <stdio.h>

#define Uses_MsgBox
#define Uses_TFilterValidator
#include <tv.h>

#define Uses_TULongRangeValidator
#include <custmval.h>

// Valid filter characters
const char * _NEAR TULongRangeValidator::validUnsignedChars = "0123456789";

// Error message string
const char * _NEAR TULongRangeValidator::errorMsg =
    "Value not in the range %lu to %lu";

TULongRangeValidator::TULongRangeValidator(unsigned long aMin,
  unsigned long aMax) :
    TFilterValidator( 0 ), min(aMin), max(aMax)
{
    // Assume that the user will want a numeric data type returned.
    options |= voTransfer;

    validChars = newStr(TULongRangeValidator::validUnsignedChars);
}

void TULongRangeValidator::error()
{
    messageBox(mfError | mfOKButton, errorMsg, min, max);
}

Boolean TULongRangeValidator::isValid(const char* s)
{
    long value;

    if(TFilterValidator::isValid(s))
        if(sscanf(s,"%lu", &value) != EOF)
            if(value >= min && value <= max)
                return True;

    return False;
}

ushort TULongRangeValidator::transfer(char* s, void* buffer, TVTransfer flag)
{
    long value;

    if(options & voTransfer)
    {
        switch(flag)
        {
            case vtGetData:
                sscanf(s, "%lu", &value);
                *(unsigned long *)buffer = value;
                break;

            case vtSetData:
               sprintf(s, "%lu", *(unsigned long *)buffer);
               break;
        }
        return sizeof(long);
    }

    return 0;
}

#if !defined(NO_STREAMABLE)

TULongRangeValidator::TULongRangeValidator(StreamableInit s) : TFilterValidator(s)
{
}

TStreamable *TULongRangeValidator::build()
{
    return new TULongRangeValidator( streamableInit );
}

void TULongRangeValidator::write(opstream& os)
{
    TFilterValidator::write( os );
    os << min << max;
}

void* TULongRangeValidator::read( ipstream& is )
{
    TFilterValidator::read( is );
    is >> min >> max;
    return this;
}

#endif
